import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_screenutil/screenutil_init.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/widgets/post.dart';

import 'home_page.dart';

class MyFeedPage extends StatefulWidget {
  const MyFeedPage({
    Key key,
    @required this.currentUser,
    @required this.saveId,
    @required this.likeId,
    @required this.dislikeId,
  }) : super(key: key);
  final UserModel currentUser;
  final int saveId;
  final int likeId;
  final int dislikeId;
  @override
  _MyFeedPageState createState() => _MyFeedPageState();
}

class _MyFeedPageState extends State<MyFeedPage>
    with SingleTickerProviderStateMixin {
  FeedCubit feedCubit;
  List<PostModel> postsList = [];
  List<PostModel> favoritePostsList = [];
  TabController _controller;
  var _scrollController = ScrollController();
  bool fixedScroll;

  getMyFeedPost() async {
    await feedCubit.feedRepository
        .fetchPostsByUserId(this.widget.currentUser.id)
        .then((myPosts) => setState(
              () {
                myPosts.sort((a, b) => a.createdAt.compareTo(b.createdAt));
                postsList = myPosts.reversed.toList();
              },
            ));
  }

  getMyFavoriteFeedPost() async {
    List<PostModel> posts = [];
    await feedCubit.feedRepository
        .fetchFavoritePosts(this.widget.currentUser.id, this.widget.saveId)
        .then((value) => {
              value.forEach((e) {
                posts.add(e.post);
              }),
              setState(
                () {
                  favoritePostsList = posts;
                },
              )
            });
  }

  @override
  void initState() {
    _controller = new TabController(length: 2, vsync: this);
    feedCubit = BlocProvider.of<FeedCubit>(context);
    getMyFeedPost();
    getMyFavoriteFeedPost();
    _scrollController.addListener(_onScroll);
    super.initState();
  }

  void _onScroll() {
    if (_scrollController.position.pixels ==
        _scrollController.position.maxScrollExtent) {
      getMyFeedPost();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(393.0, 534.0),
      allowFontScaling: false,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return MaterialApp(
            home: Scaffold(
                backgroundColor: Color(0xffE8ECEF),
                appBar: PreferredSize(
                    preferredSize: Size.fromHeight(70.0),
                    child: AppBar(
                      backgroundColor: Color(0xFF083356),
                      leading: GestureDetector(
                          onTap: () {
                            Navigator.of(context).push(MaterialPageRoute(
                                builder: (context) => HomePage(
                                      about: "CTO",
                                      firstName: "Rachel",
                                      lastName: "Broner",
                                      profileImg:
                                          "https://res.cloudinary.com/phonedo/image/upload/v1614869100/shir_fdnec1.jpg",
                                      token:
                                          "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJpYXQiOjE2MTQxNjA0NDcsImV4cCI6MTYxNjY2MDQ0Nywicm9sZXMiOlsiUk9MRV9VU0VSIiwiUk9MRV9CVVNJTkVTU19BQ0NPVU5UX0FETUlOIl0sImlkIjoiN2EzM2FkYTMtMmFiMi00ZDRhLWIwOTgtNzkyZmRkMzM0NTE4IiwidHlwZSI6IkJ1c2luZXNzQWNjb3VudFVzZXIiLCJYQXBpVG9rZW4iOiJCQS01YzYyNmNlMWVmM2U1M2U4MjM0Y2YzOTcwOTk1MzA5MCJ9.TLjKkXatKB3BXp5M2Kghn6dZ6p2uoK5mgDyOMGPqBUDsyPQ6WIoU2baEQ5U2kClVU0zO1ouyskbRQcq7LHQE6oTupbGtlJ_HcGiq_7fh5WUfNFyKAOMRUG5cLtDOXCNF_mzRmDotLOpFBvBWLkUJmVrRD-bolZPH4t3EuWgasgbXgrcOEg3axdjgNqjoTo-fxvg1nrhi4MgSGvxyVKy92KTbSOk2xZ0zePMlAjpP5TMiNCQtlLWZeOBidl7g7BkcLYQ-NZ-fTCryQ2edZ_XscIDXFcPSwPh8KZchiNCaEVATRoXFtSQMGeixPV4gw76DGwpBkmCPbg4vZFt2RC_-292YIMxIZuxubYApTl-9Ul2tSOGQKzuQNy-93gZk4Ofkurv4SEcvqCm2sLHiVnhLjzVjmovtcUT9FlkJFoWJ5ahRsHY23VBZ2ibiwBhZljcql5bFCSp-TL62Nc2gIRo-MFDrOS_NOZgm7WvKIozLDWDc11EX0_iPGF3LccP7fKOP2PD60Nyi7WXCMY4JWLAg9edsjKQECjPoEMqW0rjzdNDTPwM-Tg51HoDTauieGCy91GnLP-q-BdSvxesbxkwSizQm_qONuTBmqufiHh7m5dQ7EGj1eazsLFx76S3_EH7eR0U9S1dUCmT8kg7DF6cRC39AJK-T7Rn3dPp9mlP96bo",
                                    )));
                          },
                          child: Padding(
                            padding:
                                EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                            child: Icon(Icons.arrow_back, color: Colors.white),
                          )),
                      title: Padding(
                        padding: EdgeInsets.only(
                            right: ScreenUtil().setWidth(50),
                            top: ScreenUtil().setWidth(9)),
                        child: Center(
                            child: Text(
                          "My Feed",
                          style: TextStyle(color: Colors.white),
                        )),
                      ),
                    )),
                body: _buildContent(this.widget.currentUser)),
          );
        },
      ),
    );
  }

  _buildContent(currentUser) {
    return Stack(
      children: [
        _buildTabBarView(),
        Container(
            color: Color(0xffE8ECEF),
            height: ScreenUtil().setHeight(250),
            child: Column(
              children: [
                _buildProfile(currentUser),
                _buildDetails(),
                _activityButton(),
                _buildTabBar(),
              ],
            )),
      ],
    );
  }

  _buildProfile(currentUser) {
    return Container(
        color: Colors.white,
        child: Padding(
            padding: EdgeInsets.only(
              left: ScreenUtil().setHeight(17),
              top: ScreenUtil().setHeight(20),
              bottom: ScreenUtil().setHeight(15),
            ),
            child: Row(children: [
              Column(
                children: [
                  Container(
                    margin: EdgeInsets.only(
                      right: ScreenUtil().setWidth(10),
                    ),
                    child: ClipOval(
                      child: Image.network(
                        currentUser.profileImg,
                        width: ScreenUtil().setWidth(50),
                        height: ScreenUtil().setHeight(34),
                        fit: BoxFit.cover,
                      ),
                    ),
                  ),
                ],
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(163),
                    child: Text(
                      currentUser.firstName + " " + currentUser.lastName,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.w500,
                        color: Colors.black,
                        fontSize: ScreenUtil().setSp(17),
                        fontFamily: 'Rubik',
                      ),
                    ),
                  ),
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(165),
                    child: Text(
                      currentUser.about,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.normal,
                        color: Colors.black.withOpacity(0.4),
                        fontSize: ScreenUtil().setSp(15),
                        fontFamily: "Rubik",
                      ),
                    ),
                  ),
                ],
              ),
            ])));
  }

  _buildDetails() {
    return Container(
      color: Colors.white,
      padding: EdgeInsets.only(bottom: ScreenUtil().setHeight(5)),
      child: Row(
        children: [
          Expanded(
              child: Column(
            children: [
              Text(
                postsList.length.toString(),
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Posts"),
            ],
          )),
          Expanded(
              child: Column(
            children: [
              Text(
                "100K",
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Followers"),
            ],
          )),
          Expanded(
              child: Column(
            children: [
              Text(
                "397",
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Following"),
            ],
          ))
        ],
      ),
    );
  }

  _activityButton() {
    return Container(
        margin: EdgeInsets.only(top: ScreenUtil().setHeight(2)),
        color: Colors.white,
        height: ScreenUtil().setHeight(80),
        alignment: Alignment.center,
        child: SizedBox(
          height: ScreenUtil().setHeight(30),
          width: ScreenUtil().setHeight(245),
          child: Container(
            decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      color: Colors.black, width: 1, style: BorderStyle.solid),
                  borderRadius: BorderRadius.circular(5),
                )),
            child: FlatButton(
              onPressed: null,
              child: Text('View all activities',
                  style: TextStyle(color: Colors.black)),
            ),
          ),
        ));
  }

  _buildTabBar() {
    return Row(children: [
      Expanded(
          child: Column(children: <Widget>[
        Container(
            margin: EdgeInsets.only(top: ScreenUtil().setHeight(7)),
            color: Colors.white,
            height: ScreenUtil().setHeight(30),
            child: new TabBar(
              labelColor: Color(0xFFE083356),
              unselectedLabelColor: Color(0xFF083356),
              labelStyle: TextStyle(fontWeight: FontWeight.bold),
              unselectedLabelStyle: TextStyle(fontWeight: FontWeight.normal),
              indicatorColor: Color(0xFFE083356),
              indicatorWeight: 5.0,
              controller: _controller,
              tabs: [
                Tab(
                  text: "Posts",
                ),
                Tab(text: "Favorite"),
              ],
            )),
        Container(
          height: ScreenUtil().setHeight(17),
          margin: EdgeInsets.only(bottom: ScreenUtil().setHeight(8)),
          color: Colors.white,
        ),
      ]))
    ]);
  }

  _buildTabBarView() {
    return TabBarView(
      controller: _controller,
      children: <Widget>[
        _buildList(postsList, "Posts"),
        _buildList(favoritePostsList, "Favorite"),
      ],
    );
  }

  _buildList(listToShow, type) {
    return Stack(children: [
      // _buildTabBarView(),
      // Container(
      //     color: Color(0xffE8ECEF),
      //     height: ScreenUtil().setHeight(250),
      //     child: Column(
      //       children: [
      //         _buildProfile(currentUser),
      //         _buildDetails(),
      //         _activityButton(),
      //         _buildTabBar(),
      //       ],
      // )),
      RefreshIndicator(
        onRefresh: () {
          return getMyFeedPost();
        },
        child: ListView.separated(
          cacheExtent: 1,
          itemCount: listToShow.length + 1,
          itemBuilder: (BuildContext context, int index) {
            if (index == 0) {
              return Container(
                height: ScreenUtil().setHeight(230),
                color: Color(0xffE8ECEF),
              );
            }
            PostModel post = listToShow[index - 1];
            return FeedPost(
              postData: post,
              likeId: this.widget.likeId,
              dislikeId: this.widget.dislikeId,
              saveId: this.widget.saveId,
              currentUser: this.widget.currentUser,
              images: post.attachedUrl.split(","),
              commentsLength: 0,
            );
          },
          separatorBuilder: (BuildContext context, int index) =>
              const Divider(),
        ),
      ),
    ]);
  }
}
