import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/widgets/post.dart';

import 'home_page.dart';

class PartnerFeedPage extends StatefulWidget {
  const PartnerFeedPage({
    Key key,
    @required this.partnerUser,
    @required this.currentUser,
    @required this.saveId,
    @required this.likeId,
    @required this.dislikeId,
  }) : super(key: key);
  final UserModel partnerUser;
  final UserModel currentUser;
  final int saveId;
  final int likeId;
  final int dislikeId;
  @override
  _PartnerFeedPageState createState() => _PartnerFeedPageState();
}

class _PartnerFeedPageState extends State<PartnerFeedPage>
    with SingleTickerProviderStateMixin {
  FeedCubit feedCubit;
  List<PostModel> postsList = [];
  // List<PostModel> favoritePostsList = [];
  TabController _controller;
  var _scrollController = ScrollController();
  bool fixedScroll;

  getPartnerFeedPost() async {
    await feedCubit.feedRepository
        .fetchPostsByUserId(this.widget.partnerUser.id)
        .then((myPosts) => setState(
              () {
                myPosts.sort((a, b) => a.createdAt.compareTo(b.createdAt));
                postsList = myPosts.reversed.toList();
                ;
              },
            ));
  }

  getMyFavoriteFeedPost() async {
    List<PostModel> posts = [];
    await feedCubit.feedRepository
        .fetchFavoritePosts(this.widget.partnerUser.id, this.widget.saveId)
        .then((value) => {
              value.forEach((e) {
                posts.add(e.post);
              }),
              // setState(
              //   () {
              //     favoritePostsList = posts;
              //   },
              //)
            });
  }

  @override
  void initState() {
    _controller = new TabController(length: 2, vsync: this);
    feedCubit = BlocProvider.of<FeedCubit>(context);
    getPartnerFeedPost();
    getMyFavoriteFeedPost();
    _scrollController.addListener(_onScroll);
    super.initState();
  }

  void _onScroll() {
    if (_scrollController.position.pixels ==
        _scrollController.position.maxScrollExtent) {
      getPartnerFeedPost();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(393.0, 534.0),
      allowFontScaling: false,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return MaterialApp(
            home: Scaffold(
                backgroundColor: Color(0xffE8ECEF),
                appBar: PreferredSize(
                    preferredSize: Size.fromHeight(70.0),
                    child: AppBar(
                      backgroundColor: Color(0xFF083356),
                      leading: GestureDetector(
                          onTap: () {
                            Navigator.of(context).popAndPushNamed("/homePage");
                          },
                          child: Padding(
                            padding:
                                EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                            child: Icon(Icons.arrow_back, color: Colors.white),
                          )),
                      title: Padding(
                        padding: EdgeInsets.only(
                            right: ScreenUtil().setWidth(50),
                            top: ScreenUtil().setWidth(9)),
                        child: Center(
                            child: Text(
                          this.widget.partnerUser.firstName + " Feed",
                          style: TextStyle(color: Colors.white),
                        )),
                      ),
                    )),
                body: _buildContent(this.widget.partnerUser)),
          );
        },
      ),
    );
  }

  _buildContent(partnerUser) {
    return Stack(
      children: [
        //  _buildTabBarView(),
        Container(
            color: Color(0xffE8ECEF),
            height: ScreenUtil().setHeight(450),
            child: Column(
              children: [
                _buildProfile(partnerUser),
                _buildDetails(),
                _activityButton(),
                _buildTabBar(),
              ],
            )),
      ],
    );
  }

  _buildProfile(partnerUser) {
    return Container(
        color: Colors.white,
        child: Padding(
            padding: EdgeInsets.only(
              left: ScreenUtil().setHeight(17),
              top: ScreenUtil().setHeight(20),
              bottom: ScreenUtil().setHeight(15),
            ),
            child: Row(children: [
              Column(
                children: [
                  Container(
                    margin: EdgeInsets.only(
                      right: ScreenUtil().setWidth(10),
                    ),
                    child: ClipOval(
                      child: Image.network(
                        partnerUser.profileImg,
                        width: ScreenUtil().setWidth(50),
                        height: ScreenUtil().setHeight(34),
                        fit: BoxFit.cover,
                      ),
                    ),
                  ),
                ],
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(163),
                    child: Text(
                      this.widget.partnerUser.firstName,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.w500,
                        color: Colors.black,
                        fontSize: ScreenUtil().setSp(17),
                        fontFamily: 'Rubik',
                      ),
                    ),
                  ),
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(165),
                    child: Text(
                      partnerUser.about,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.normal,
                        color: Colors.black.withOpacity(0.4),
                        fontSize: ScreenUtil().setSp(15),
                        fontFamily: "Rubik",
                      ),
                    ),
                  ),
                ],
              ),
            ])));
  }

  _buildDetails() {
    return Container(
      color: Colors.white,
      padding: EdgeInsets.only(bottom: ScreenUtil().setHeight(5)),
      child: Row(
        children: [
          Expanded(
              child: Column(
            children: [
              Text(
                postsList.length.toString(),
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Posts"),
            ],
          )),
          Expanded(
              child: Column(
            children: [
              Text(
                "100K",
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Followers"),
            ],
          )),
          Expanded(
              child: Column(
            children: [
              Text(
                "397",
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
              Text("Following"),
            ],
          ))
        ],
      ),
    );
  }

  _activityButton() {
    return Container(
        margin: EdgeInsets.only(top: ScreenUtil().setHeight(2)),
        color: Colors.white,
        height: ScreenUtil().setHeight(80),
        alignment: Alignment.center,
        child: SizedBox(
          height: ScreenUtil().setHeight(30),
          width: ScreenUtil().setHeight(245),
          child: Container(
            decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      color: Colors.black, width: 1, style: BorderStyle.solid),
                  borderRadius: BorderRadius.circular(5),
                )),
            child: FlatButton(
              onPressed: null,
              child: Text("Follow" + " " + this.widget.partnerUser.firstName,
                  style: TextStyle(color: Colors.black)),
            ),
          ),
        ));
  }

  _buildTabBar() {
    return Row(children: [
      Expanded(
          child: Column(children: <Widget>[
        Container(
          margin: EdgeInsets.only(top: ScreenUtil().setHeight(2)),
          color: Colors.white,
          height: ScreenUtil().setHeight(30),
        ),
        Container(
          height: ScreenUtil().setHeight(150),
          margin: EdgeInsets.only(bottom: ScreenUtil().setHeight(8)),
          color: Colors.white,
        ),
      ]))
    ]);
  }
}
