import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_screenutil/screenutil_init.dart';
import 'package:number_display/number_display.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/models/report_sub_type_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/widgets/loadingButtonIndicator.dart';
import 'package:feed_plugin/widgets/post.dart';

import 'home_page.dart';

class UserPost extends StatefulWidget {
  const UserPost({Key key, @required this.userData, @required this.currentUser})
      : super(key: key);

  final UserModel userData;
  final UserModel currentUser;
  @override
  _UserPostState createState() => _UserPostState();
}

class _UserPostState extends State<UserPost> {
  FeedCubit feedCubit;
  var _scrollController = ScrollController();
  @override
  void initState() {
    _scrollController.addListener(_onScroll);
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitDown,
      DeviceOrientation.portraitUp,
    ]);
    feedCubit = BlocProvider.of<FeedCubit>(context);
    super.initState();
  }

  void _onScroll() {
    if (_scrollController.position.pixels ==
        _scrollController.position.maxScrollExtent) {
      _getPostsList();
    }
  }

  final display = createDisplay(
    length: 4,
    separator: ',',
    decimal: 2,
  );
  double defaultScreenWidth = 393.0;
  double defaultScreenHeight = 534.0;
  List<PostModel> postsList = [];
  List<ReportSubTypeModel> reportsSubTypeList = [];
  int likeId;
  int dislikeId;
  int saveId;

  Future<List<PostModel>> _getPostsList() async {
    await getLikeSubId();
    postsList = await feedCubit.feedRepository
        .fetchPostsByUserId(this.widget.userData.id);

    return postsList;
  }

  Future getLikeSubId() async {
    reportsSubTypeList = await feedCubit.feedRepository.fetchAllSubType();
    for (var i = 0; i < reportsSubTypeList.length; i++) {
      if (likeId == null || dislikeId == null || saveId == null) {
        if (reportsSubTypeList[i].type == "Like") {
          likeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Dislike") {
          dislikeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Save") {
          saveId = reportsSubTypeList[i].id;
        }
      } else {
        break;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    var agentName = (this.widget.userData.firstName).toString() +
        " " +
        (this.widget.userData.lastName).toString();
    return ScreenUtilInit(
      designSize: Size(defaultScreenWidth, defaultScreenHeight),
      allowFontScaling: false,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return MaterialApp(
            home: Scaffold(
              appBar: PreferredSize(
                  preferredSize: Size.fromHeight(70.0),
                  child: AppBar(
                    backgroundColor: Color(0xFF083356),
                    leading: GestureDetector(
                        onTap: () {
                          Navigator.of(context).push(MaterialPageRoute(
                              builder: (context) => HomePage(
                                    about: "CTO",
                                    firstName: "Rachel",
                                    lastName: "Broner",
                                    profileImg:
                                        "https://res.cloudinary.com/phonedo/image/upload/v1614869100/shir_fdnec1.jpg",
                                    token:
                                        "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJpYXQiOjE2MTQxNjA0NDcsImV4cCI6MTYxNjY2MDQ0Nywicm9sZXMiOlsiUk9MRV9VU0VSIiwiUk9MRV9CVVNJTkVTU19BQ0NPVU5UX0FETUlOIl0sImlkIjoiN2EzM2FkYTMtMmFiMi00ZDRhLWIwOTgtNzkyZmRkMzM0NTE4IiwidHlwZSI6IkJ1c2luZXNzQWNjb3VudFVzZXIiLCJYQXBpVG9rZW4iOiJCQS01YzYyNmNlMWVmM2U1M2U4MjM0Y2YzOTcwOTk1MzA5MCJ9.TLjKkXatKB3BXp5M2Kghn6dZ6p2uoK5mgDyOMGPqBUDsyPQ6WIoU2baEQ5U2kClVU0zO1ouyskbRQcq7LHQE6oTupbGtlJ_HcGiq_7fh5WUfNFyKAOMRUG5cLtDOXCNF_mzRmDotLOpFBvBWLkUJmVrRD-bolZPH4t3EuWgasgbXgrcOEg3axdjgNqjoTo-fxvg1nrhi4MgSGvxyVKy92KTbSOk2xZ0zePMlAjpP5TMiNCQtlLWZeOBidl7g7BkcLYQ-NZ-fTCryQ2edZ_XscIDXFcPSwPh8KZchiNCaEVATRoXFtSQMGeixPV4gw76DGwpBkmCPbg4vZFt2RC_-292YIMxIZuxubYApTl-9Ul2tSOGQKzuQNy-93gZk4Ofkurv4SEcvqCm2sLHiVnhLjzVjmovtcUT9FlkJFoWJ5ahRsHY23VBZ2ibiwBhZljcql5bFCSp-TL62Nc2gIRo-MFDrOS_NOZgm7WvKIozLDWDc11EX0_iPGF3LccP7fKOP2PD60Nyi7WXCMY4JWLAg9edsjKQECjPoEMqW0rjzdNDTPwM-Tg51HoDTauieGCy91GnLP-q-BdSvxesbxkwSizQm_qONuTBmqufiHh7m5dQ7EGj1eazsLFx76S3_EH7eR0U9S1dUCmT8kg7DF6cRC39AJK-T7Rn3dPp9mlP96bo",
                                  )));
                        },
                        child: Padding(
                          padding:
                              EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                          child: Icon(Icons.arrow_back, color: Colors.white),
                        )),
                    title: Padding(
                      padding: EdgeInsets.only(
                          left: ScreenUtil().setWidth(10),
                          top: ScreenUtil().setWidth(9)),
                      child: Text(
                        agentName.toString() + " post",
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  )),
              body: FutureBuilder(
                future: _getPostsList(),
                builder: (context, projectSnap) {
                  if (projectSnap.connectionState == ConnectionState.waiting ||
                      projectSnap.connectionState == ConnectionState.none) {
                    return Center(child: LoadingIndicatorDark());
                  }
                  return Stack(children: [
                    RefreshIndicator(
                      onRefresh: () {
                        return _getPostsList();
                      },
                      child: ListView.separated(
                        cacheExtent: 9999,
                        padding: EdgeInsets.all(ScreenUtil().setWidth(8)),
                        itemCount: postsList.length,
                        itemBuilder: (BuildContext context, int index) {
                          PostModel post = projectSnap.data[index];
                          return FeedPost(
                            postData: post,
                            likeId: likeId,
                            dislikeId: dislikeId,
                            saveId: saveId,
                            currentUser: this.widget.currentUser,
                            images: post.attachedUrl.split(","),
                            commentsLength: 0,
                          );
                        },
                        separatorBuilder: (BuildContext context, int index) =>
                            const Divider(),
                      ),
                    ),
                  ]);
                },
              ),
            ),
          );
        },
      ),
    );
  }
}
